//
//  CFOldStylePList.h
//  GlyphsCore
//
//  Created by Georg Seifert on 16.02.19.
//  Copyright © 2019 schriftgestaltung.de. All rights reserved.
//

#ifndef CFOldStylePList_h
#define CFOldStylePList_h

typedef struct {
	unsigned char *curr;
	const unsigned char *end;
	const unsigned char *begin;
	bool inQuotedString; // this is used to be able to parse userdata in nodestrings
	CFErrorRef error;
	CFAllocatorRef allocator;
	CFOptionFlags mutabilityOption;
	CFMutableSetRef stringSet;  // set of all strings involved in this parse; allows us to share non-mutable strings in the returned plist
} GSPlistParseInfo;

BOOL GSCheckKey(GSPlistParseInfo *pInfo, const char *key, int length);
BOOL GSCreatePlistParseInfoWithFile(NSString* filePath, GSPlistParseInfo *pInfo);
void GSClosePlistParseInfoWithFile(GSPlistParseInfo *pInfo);

CFErrorRef GSPropertyListCreateError(CFIndex code, NSString *debugFormat, ...);
CFErrorRef GSPropertyListCreateErrorDetails(CFIndex code, NSString *description, NSString *suggestion);
UInt32 GSLineNumberStrings(GSPlistParseInfo *pInfo);

NSDictionary *GSParseStringDict(NSString *string, NSError **error);

CFTypeRef GSParsePlistFile(NSString *filePath, NSError **error) CF_RETURNS_RETAINED;

NSDictionary *GSParseFileDict(NSString *filePath, NSError **error);
NSDictionary *GSParsePlistDict(GSPlistParseInfo *pInfo);
NSMutableDictionary *GSParsePlistDictMutable(GSPlistParseInfo *pInfo);

NSArray *GSParseFileArray(NSString *filePath, NSError **error);

NSArray *GSParsePlistArray(GSPlistParseInfo *pInfo) __attribute__((ns_returns_retained));
NSString *GSParsePlistKey(GSPlistParseInfo *pInfo) __attribute__((ns_returns_retained));
NSString *GSParsePlistText(GSPlistParseInfo *pInfo) __attribute__((ns_returns_retained));
NSDate *GSParsePlistDate(GSPlistParseInfo *pInfo) __attribute__((ns_returns_retained));
CGFloat GSParsePlistFloat(GSPlistParseInfo *pInfo);
NSInteger GSParsePlistInteger(GSPlistParseInfo *pInfo);
BOOL GSParsePlistBool(GSPlistParseInfo *pInfo);
NSPoint GSParsePlistPoint(GSPlistParseInfo *pInfo);
BOOL GSParsePlistFloatListCallback(GSPlistParseInfo *pInfo, void (^callback)(CGFloat result));
BOOL GSParsePlistFloatList(GSPlistParseInfo *pInfo, CGFloat *values, NSInteger *count);
BOOL GSParsePlistTransform(GSPlistParseInfo *pInfo, NSAffineTransformStruct *transform);
NSObject *GSParsePlistObject(GSPlistParseInfo *pInfo);
NSColor *GSParsePlistColor(GSPlistParseInfo *pInfo);

BOOL GSAdvanceToNonSpace(GSPlistParseInfo *pInfo);

NSString *GSParseQuotedText(GSPlistParseInfo *pInfo);
#define skipWhiteSpace(pInfo) if (!GSAdvanceToNonSpace(pInfo)) return 0;

#endif /* CFOldStylePList_h */
